"""Module contains main API to work with pytest-emoji-output plugin."""
import sys
from enum import Enum
from typing import Tuple, Optional
from _pytest.config import Config
from _pytest.config.argparsing import Parser, OptionGroup
from _pytest.reports import TestReport
from attr import dataclass


class _Emoji(Enum):
    """The class represents an emoji element."""

    HOLY: str = "😇"
    HELLISH: str = "😡"
    SILENT: str = "😶"

    def __str__(self) -> str:
        """Returns emoji string value."""
        return self.value


@dataclass(frozen=True, slots=True)
class _TestStatus:
    """The class represents test status element."""

    outcome: str
    short: str
    long: str


def pytest_addoption(parser: Parser) -> None:
    """Turns on emoji parser feature.

    Args:
        parser (Parser): cli parser
    """
    group: OptionGroup = parser.getgroup("emoji")
    group.addoption(
        "--emoji-out",
        "--eo",
        action="store_true",
        help="Adds emoji to pytest results",
    )


def pytest_report_header(config: Config) -> Optional[str]:  # type: ignore
    """Adds header to pytest runner.

    Args:
        config (Config): configuration option
    """
    if config.getoption("emoji_out"):
        return (
            f"Running on {sys.platform} platform: "
            f"{'{}.{}.{}'.format(*sys.version_info[:3])} python version"
        )


def pytest_report_teststatus(  # type: ignore
    report: TestReport, config: Config
) -> Optional[Tuple[str, str, str]]:
    """Turn failures into opportunities.

    Args:
        report (TestReport): pytest report item
        config (Config): pytest configuration item
    """
    if report.when == "call" and config.getoption("emoji_out"):
        if report.passed:
            passed: _TestStatus = _TestStatus(
                outcome=report.outcome,
                short=str(_Emoji.HOLY),
                long=f"{_Emoji.HOLY} Yes sir, it is passed",
            )
            return passed.outcome, passed.short, passed.long
        if report.failed:
            failed: _TestStatus = _TestStatus(
                outcome=report.outcome,
                short=str(_Emoji.HELLISH),
                long=f"{_Emoji.HELLISH} Oh crap, it is failed",
            )
            return failed.outcome, failed.short, failed.long
        if report.skipped:
            skipped: _TestStatus = _TestStatus(
                outcome=report.outcome,
                short=str(_Emoji.SILENT),
                long=f"{_Emoji.SILENT} Nevermind, it is skipped",
            )
            return skipped.outcome, skipped.short, skipped.long



"""Package provides API for testing output with emoji."""

__version__: str = "0.2.0"
__package_name__: str = "pytest-emoji-output"
__author__: str = "Volodymyr Yahello"
__email__: str = "vyahello@gmail.com"
__license__: str = "MIT"



import os
import codecs
from typing import Sequence, IO
from setuptools import setup, find_packages
from plugin import (
    __author__,
    __email__,
    __license__,
    __package_name__,
    __version__,
)


def __read(filename: str) -> str:
    """Reads filename.

    Args:
        filename: name of a file to be read
    """
    return codecs.open(
        os.path.join(os.path.dirname(__file__), filename), encoding="utf-8"
    ).read()


def __description() -> str:
    """Returns project description."""
    return __read("README.md")


def __requirements() -> Sequence[str]:
    """Returns requirements sequence."""
    with open("requirements.txt", "r") as requirements:  # type: IO[str]
        return tuple(map(str.strip, requirements.readlines()))


def __packages_to_not_install() -> Sequence[str]:
    """Returns a list of packages to be not installed."""
    return "*.tests", "*.tests.*", "tests.*", "tests"


if __name__ == "__main__":
    setup(
        name=__package_name__,
        version=__version__,
        author=__author__,
        author_email=__email__,
        maintainer=__author__,
        maintainer_email=__email__,
        license=__license__,
        url=f"https://github.com/vyahello/{__package_name__}",
        description="Pytest plugin to represent test output with emoji support",
        long_description=__description(),
        long_description_content_type="text/markdown",
        py_modules=["plugin.emoji"],
        packages=find_packages(exclude=__packages_to_not_install()),
        include_package_data=True,
        install_requires=__requirements(),
        classifiers=[
            "Development Status :: 4 - Beta",
            "Framework :: Pytest",
            "Intended Audience :: Developers",
            "Topic :: Software Development :: Testing",
            "Programming Language :: Python",
            "Programming Language :: Python :: 3.6",
            "Programming Language :: Python :: 3.7",
            "Programming Language :: Python :: 3.8",
            "Programming Language :: Python :: 3.9",
            "Programming Language :: Python :: 3.10",
            "Programming Language :: Python :: Implementation :: CPython",
            "Operating System :: OS Independent",
            f"License :: OSI Approved :: {__license__} License",
        ],
        python_requires=">=3.6",
        entry_points={"pytest11": ("emoji = plugin.emoji",)},
    )
